const EXPORTED_SYMBOLS = ["TwitterHultmann"];

const Cc = Components.classes;
const Ci = Components.interfaces;

const IGNORE_BLOCKING_NAME = "twitter-flag-ignore-blocking";
const IGNORE_BLOCKING_VALUE = "ok";
const ORIGINAL_DATA = "twitter-css-backup";

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
Components.utils.import("resource://twitter-hltmnn-modules/util.js");


const TwitterHultmann = {
  onToolbarButtonCommand: function(evt) {
    util.log(evt.target.tagName + "--" + evt.currentTarget.tagName);
  }
};


const httpObservers = {
  response: {
    QueryInterface: XPCOMUtils.generateQI([Ci.nsIObserver]),

    // nsIObserver
    observe: function(aSubject, aTopic, aData) {
      var httpChannel = aSubject.QueryInterface(Ci.nsIHttpChannel);

      if (httpChannel.contentLength < Prefs.bgMax) {
        //util.log("contentLength " + httpChannel.contentLength + "\n"+httpChannel.URI.spec);
        return;
      }

      if (httpChannel.contentType.indexOf("image/") !== 0) {
        //util.log("content-type " + httpChannel.contentType + "\n"+httpChannel.URI.spec);
        return;
      }


      var win = fromTwitter(httpChannel);
      if (win === null) {
        //util.log("fromTwitter=false " + httpChannel.URI.spec);
        return;
      }


      if (win.document.getUserData(IGNORE_BLOCKING_NAME) === IGNORE_BLOCKING_VALUE) {
        return;
      }


      var style = win.getComputedStyle(win.document.querySelector("body"), null);
      var userData = {
        "image":      style.getPropertyValue("background-image"),
        "repeat":     style.getPropertyValue("background-repeat"),
        "color":      style.getPropertyValue("background-color"),
        "attachment": style.getPropertyValue("background-attachment")
      };

      var url0 = httpChannel.URI.spec;
      var url1 = 'url("' + url0 + '")';
      var url2 = 'url('  + url0 +  ')';
      var img = userData["image"];
      if (img === url1 || img === url2) {
        cancelLoad(httpChannel, userData, win);
      }

    }
  }
};


function cancelLoad(httpChannel, userData, win) {
  if (userData["image"].indexOf("url(data:") === 0) {
    util.log("LOAD2  " + 'body background-image: url(data:...' + "\nrequest: " + httpChannel.URI.spec);
    //return;
  }

  util.log("cancel loading " + httpChannel.contentLength + " > " + Prefs.bgMax + "\n" + httpChannel.URI.spec);
  httpChannel.cancel(Components.results.NS_ERROR_ABORT);


  win.document.setUserData(ORIGINAL_DATA, JSON.stringify(userData), null);

  var s = win.document.body.style;
  s.backgroundImage = "url(" + buildImg(win.document, httpChannel.contentLength, userData["color"]) + ")";
  s.backgroundAttachment = "scroll";
  s.backgroundRepeat = "no-repeat";

  win.document.addEventListener("click", function(evt) {
    if (evt.button !== 0 || evt.target.tagName !== "BODY") {
      return;
    }
    var doc = evt.target.ownerDocument;
    var userData = JSON.parse(doc.getUserData(ORIGINAL_DATA));
    doc.setUserData(IGNORE_BLOCKING_NAME, IGNORE_BLOCKING_VALUE, null);

    var s = doc.body.style;
    s.backgroundRepeat = userData["repeat"];
    s.backgroundImage = userData["image"];
    s.backgroundAttachment = userData["attachment"];
    s.backgroundColor = userData["color"];
  }, false)

}


function fromTwitter(httpChannel) {
  var domWindow = getChannelWindow(httpChannel);
  if (domWindow === null) {
    util.log("no win " + httpChannel.URI.spec);
    return null;
  }

  var uri = domWindow.document.documentURIObject;
  switch (uri.scheme) {
    case "http":
    case "https":
      break;
    default:
      return null;
  }

  if (uri.host !== "twitter.com") {
    //util.log("e " + domWindow.document.documentURIObject.spec + "\n" + httpChannel.URI.spec);
    return null;
  }

  return domWindow;
}


function buildImg(doc, size, bgColor) {
  util.log("bgcolor=" + bgColor);
  var cnv = doc.createElement("canvas");
  cnv.width = 200;
  cnv.height = 400;


  var ctx = cnv.getContext("2d");

  ctx.rotate(-70 * Math.PI / 180);
  ctx.font = "bold 2.3em Verdana, Helvetica, sans-serif";
  ctx.fillStyle = "rgba(255, 255, 255, 0.9)";
  ctx.shadowOffsetX = 0;
  ctx.shadowOffsetY = 0;
  ctx.shadowBlur = 7;
  ctx.shadowColor = "rgba(0, 0, 0, 0.9)";

  var txt = util.getText("blocked");
  var w = ctx.measureText(txt).width;
  ctx.fillText(txt, -w/2 - 170, 140);

  var txt = util.getText("click", Math.round(size / 1024));
  var w = ctx.measureText(txt).width;
  ctx.fillText(txt, -w/2 - 170, 170);

  return cnv.toDataURL("image/png");
}


function getChannelWindow(channel) {
  if (channel.notificationCallbacks) {
    try {
      return channel
              .notificationCallbacks
              .getInterface(Ci.nsILoadContext)
              .associatedWindow;
    } catch (ex) {
      //util2.logEx("channel.notificationCallbacks ", channel.notificationCallbacks, channel.URI.spec, ex);
    }
  }

  if (channel.loadGroup && channel.loadGroup.notificationCallbacks) {
    try {
      return channel
              .loadGroup
              .notificationCallbacks
              .getInterface(Ci.nsILoadContext)
              .associatedWindow;
    } catch (ex) {
      //util2.logEx("channel.loadGroup", channel.loadGroup, channel.URI.spec, ex);
    }
  }

  return null; // e.g. <link rel=prefetch> <link rel=next> ...
}


const Prefs = {
  _bgMax: 0,
  _branch: null,

  init: function() {
    this._branch = Cc["@mozilla.org/preferences-service;1"]
                    .getService(Ci.nsIPrefService)
                    .getBranch("extensions.twitter@hultmann.");
    this._branch.QueryInterface(Ci.nsIPrefBranch2);
    this._branch.addObserver("", this, false);
    this.update();
  },

  get bgMax() {
    return this._bgMax;
  },

  update: function() {
    this._bgMax = this._branch.getIntPref("web.backgroundBlocker.size") * 1024;
    util.log('pref! ' + this._bgMax);
  },

  QueryInterface: XPCOMUtils.generateQI([Ci.nsIObserver]),
  observe: function(aSubject, aTopic, aData) {
    this.update();
  }

};


Prefs.init();
util.setHttpObservers.enable(httpObservers.response);
